@@ -43,15 +43,12 @@ public:
         : QSortFilterProxyModel(parent)
     {
     }
-
-protected:
-    bool lessThan(const QModelIndex &source_left, const QModelIndex &source_right) const override;
 };
 
 KCMKScreen::KCMKScreen(QObject *parent, const KPluginMetaData &data)
     : KQuickManagedConfigModule(parent, data)
 {
-    qmlRegisterAnonymousType<OutputModel>("org.kde.private.kcm.screen", 1);
+    qmlRegisterUncreatableType<OutputModel>("org.kde.private.kcm.kscreen", 1, 0, "OutputModel", QStringLiteral("For enums"));
     qmlRegisterType<KScreen::Output>("org.kde.private.kcm.kscreen", 1, 0, "Output");
     qmlRegisterUncreatableType<KCMKScreen>("org.kde.private.kcm.kscreen", 1, 0, "KCMKScreen", QStringLiteral("For InvalidConfig enum"));
     Log::instance();
@@ -59,7 +56,6 @@ KCMKScreen::KCMKScreen(QObject *parent, const KPluginMetaData &data)
     setButtons(Apply);
 
     m_outputProxyModel = new ScreenSortProxyModel(this);
-    m_outputProxyModel->sort(0);
 
     m_loadCompressor = new QTimer(this);
     m_loadCompressor->setInterval(1000);
@@ -186,6 +182,8 @@ void KCMKScreen::doSave()
         exportGlobalScale();
     }
 
+    m_configHandler->prepareForSave();
+
     m_configHandler->writeControl();
 
     // Store the current config, apply settings. Block until operation is
@@ -226,12 +224,6 @@ void KCMKScreen::doSave()
         // enough time to change configuration.
         QTimer::singleShot(1000, this, updateInitialData);
     }
-
-    if (m_needsKwinConfigReload) {
-        m_needsKwinConfigReload = false;
-        QDBusMessage message = QDBusMessage::createSignal("/KWin", "org.kde.KWin", "reloadConfig");
-        QDBusConnection::sessionBus().send(message);
-    }
 }
 
 bool KCMKScreen::backendReady() const
@@ -529,10 +521,6 @@ bool KCMKScreen::xwaylandClientsScaleSupported() const
 
 void KCMKScreen::setAllowTearing(bool allow)
 {
-    if (KWinCompositingSetting::self()->allowTearing() == allow) {
-        return;
-    }
-    m_needsKwinConfigReload = true;
     KWinCompositingSetting::self()->setAllowTearing(allow);
     Q_EMIT changed();
 }
@@ -556,15 +544,9 @@ bool KCMKScreen::multipleScreensAvailable() const
     return m_outputProxyModel->rowCount() > 1;
 }
 
-bool ScreenSortProxyModel::lessThan(const QModelIndex &source_left, const QModelIndex &source_right) const
+void KCMKScreen::startHdrCalibrator(const QString &outputName)
 {
-    const bool leftEnabled = source_left.data(OutputModel::EnabledRole).toBool();
-    const bool rightEnabled = source_right.data(OutputModel::EnabledRole).toBool();
-
-    if (leftEnabled != rightEnabled) {
-        return leftEnabled;
-    }
-    return QSortFilterProxyModel::lessThan(source_left, source_right);
+    QProcess::startDetached("hdrcalibrator", {outputName});
 }
 
 #include "kcm.moc"
