@@ -94,6 +94,8 @@ public:
     Q_INVOKABLE void setStopUpdatesFromBackend(bool value);
     Q_INVOKABLE void updateFromBackend();
 
+    Q_INVOKABLE void startHdrCalibrator(const QString &outputName);
+
 Q_SIGNALS:
     void backendReadyChanged();
     void backendError();
@@ -135,7 +137,6 @@ private:
     bool m_settingsReverted = false;
     bool m_stopUpdatesFromBackend = false;
     bool m_configNeedsSave = false;
-    bool m_needsKwinConfigReload = false;
 
     QSortFilterProxyModel *m_outputProxyModel;
 
