@@ -93,7 +93,7 @@ QVariant OutputModel::data(const QModelIndex &index, int role) const
         QVariantList ret;
         const auto rates = refreshRates(output);
         for (const auto rate : rates) {
-            ret << i18n("%1 Hz", int(rate + 0.5));
+            ret << i18n("%1 Hz", QString::number(rate, 'f', 2));
         }
         return ret;
     }
@@ -123,6 +123,22 @@ QVariant OutputModel::data(const QModelIndex &index, int role) const
         return output->brightness();
     case ColorPowerPreference:
         return static_cast<uint32_t>(output->colorPowerPreference());
+    case DdcCiAllowedRole:
+        return output->ddcCiAllowed();
+    case MaxBitsPerColorRole:
+        return output->maxBitsPerColor();
+    case AutomaticMaxBitsPerColorLimitRole:
+        return output->automaticMaxBitsPerColorLimit();
+    case MinSupportedMaxBitsPerColorRole:
+        return output->bitsPerColorRange().min;
+    case MaxSupportedMaxBitsPerColorRole:
+        return output->bitsPerColorRange().max;
+    case PeakBrightnessOverride:
+        return output->maxPeakBrightnessOverride().value_or(output->maxPeakBrightness());
+    case OutputNameRole:
+        return output->name();
+    case EdrPolicyRole:
+        return static_cast<uint32_t>(output->edrPolicy());
     }
     return QVariant();
 }
@@ -293,6 +309,22 @@ bool OutputModel::setData(const QModelIndex &index, const QVariant &value, int r
         output.ptr->setColorPowerPreference(static_cast<KScreen::Output::ColorPowerTradeoff>(value.toUInt()));
         Q_EMIT dataChanged(index, index, {role});
         return true;
+    case DdcCiAllowedRole:
+        output.ptr->setDdcCiAllowed(value.toBool());
+        Q_EMIT dataChanged(index, index, {role});
+        return true;
+    case MaxBitsPerColorRole:
+        output.ptr->setMaxBitsPerColor(value.toUInt());
+        Q_EMIT dataChanged(index, index, {role});
+        return true;
+    case PeakBrightnessOverride:
+        output.ptr->setMaxPeakBrightnessOverride(value.toDouble());
+        Q_EMIT dataChanged(index, index, {role});
+        return true;
+    case EdrPolicyRole:
+        output.ptr->setEdrPolicy(static_cast<KScreen::Output::EdrPolicy>(value.toUInt()));
+        Q_EMIT dataChanged(index, index, {role});
+        return true;
     }
     return false;
 }
@@ -330,6 +362,14 @@ QHash<int, QByteArray> OutputModel::roleNames() const
     roles[ColorProfileSource] = "colorProfileSource";
     roles[BrightnessRole] = "brightness";
     roles[ColorPowerPreference] = "colorPowerPreference";
+    roles[DdcCiAllowedRole] = "ddcCiAllowed";
+    roles[MaxBitsPerColorRole] = "maxBitsPerColor";
+    roles[AutomaticMaxBitsPerColorLimitRole] = "automaticMaxBitsPerColorLimit";
+    roles[MinSupportedMaxBitsPerColorRole] = "minSupportedMaxBitsPerColor";
+    roles[MaxSupportedMaxBitsPerColorRole] = "maxSupportedMaxBitsPerColor";
+    roles[PeakBrightnessOverride] = "peakBrightnessOverride";
+    roles[OutputNameRole] = "name";
+    roles[EdrPolicyRole] = "edrPolicy";
     return roles;
 }
 
@@ -486,7 +526,7 @@ bool OutputModel::setEnabled(int outputIndex, bool enable)
 
 inline bool refreshRateCompare(float rate1, float rate2)
 {
-    return qAbs(rate1 - rate2) < 0.5;
+    return qAbs(rate1 - rate2) < 0.01;
 }
 
 bool OutputModel::setResolution(int outputIndex, int resIndex)
@@ -658,7 +698,7 @@ QVariantList OutputModel::resolutionsStrings(const KScreen::OutputPtr &output) c
     for (const QSize &size : resolutionList) {
         if (size.isEmpty()) {
             const QString text = i18nc("Width x height",
-                                       "%1x%2",
+                                       "%1 × %2",
                                        // Explicitly not have it add thousand-separators.
                                        QString::number(size.width()),
                                        QString::number(size.height()));
@@ -677,7 +717,7 @@ QVariantList OutputModel::resolutionsStrings(const KScreen::OutputPtr &output) c
             }
 
             const QString text = i18nc("Width x height (aspect ratio)",
-                                       "%1x%2 (%3:%4)",
+                                       "%1 × %2 (%3:%4)",
                                        // Explicitly not have it add thousand-separators.
                                        QString::number(size.width()),
                                        QString::number(size.height()),
@@ -749,11 +789,17 @@ QList<float> OutputModel::refreshRates(const KScreen::OutputPtr &output) const
 
 int OutputModel::replicationSourceId(const Output &output) const
 {
-    const KScreen::OutputPtr source = m_config->replicationSource(output.ptr);
-    if (!source) {
-        return 0;
+    if (m_config->config()->supportedFeatures() & KScreen::Config::Feature::PerOutputScaling) {
+        // on Wayland, just return the output's replication source...
+        return output.ptr->replicationSource();
+    } else {
+        // keep the old behavior for X11
+        const KScreen::OutputPtr source = m_config->replicationSource(output.ptr);
+        if (!source) {
+            return 0;
+        }
+        return source->id();
     }
-    return source->id();
 }
 
 QStringList OutputModel::replicationSourceModel(const KScreen::OutputPtr &output) const
@@ -824,6 +870,7 @@ static KScreen::ModePtr getBestMode(const KScreen::OutputPtr &output, const KScr
 bool OutputModel::setReplicationSourceIndex(int outputIndex, int sourceIndex)
 {
     if (outputIndex <= sourceIndex) {
+        // the output itself isn't in the list of the model
         sourceIndex++;
     }
     if (sourceIndex >= m_outputs.count()) {
@@ -833,47 +880,58 @@ bool OutputModel::setReplicationSourceIndex(int outputIndex, int sourceIndex)
     Output &output = m_outputs[outputIndex];
     const int oldSourceId = replicationSourceId(output);
 
-    if (sourceIndex < 0) {
-        if (oldSourceId == 0) {
-            // no change
-            return false;
+    if (m_config->config()->supportedFeatures() & KScreen::Config::Feature::PerOutputScaling) {
+        // on Wayland, we just set the replication source, and that's all
+        if (sourceIndex >= 0) {
+            output.ptr->setReplicationSource(m_outputs[sourceIndex].ptr->id());
+        } else {
+            output.ptr->setReplicationSource(0);
         }
-        m_config->setReplicationSource(output.ptr, nullptr);
-        output.ptr->setExplicitLogicalSize(QSizeF());
-        resetPosition(output);
     } else {
-        const auto source = m_outputs[sourceIndex].ptr;
-        if (oldSourceId == source->id()) {
-            // no change
-            return false;
-        }
+        // keep the old behavior for X11, to avoid breaking it
+        if (sourceIndex < 0) {
+            if (oldSourceId == 0) {
+                // no change
+                return false;
+            }
+            m_config->setReplicationSource(output.ptr, nullptr);
+            output.ptr->setExplicitLogicalSize(QSizeF());
+            resetPosition(output);
+        } else {
+            const auto source = m_outputs[sourceIndex].ptr;
+            if (oldSourceId == source->id()) {
+                // no change
+                return false;
+            }
 
-        // In replicating outputs, we don't change the source
-        // Step 1: set the destination mode, if possible
-        const auto bestMode = getBestMode(output.ptr, source);
-        if (!bestMode) {
-            return false;
-        }
-        // Step 2: reposition ans scale destination output to be centered inside the source output
-        auto sourceSize = source->currentMode()->size();
-        auto destinationSize = bestMode->size();
-        if (source->rotation() == KScreen::Output::Left || source->rotation() == KScreen::Output::Right) {
-            sourceSize = sourceSize.transposed();
-        }
-        if (output.ptr->rotation() == KScreen::Output::Left || output.ptr->rotation() == KScreen::Output::Right) {
-            destinationSize = destinationSize.transposed();
-        }
-        qreal scale = source->scale() * std::max(destinationSize.width() / qreal(sourceSize.width()), destinationSize.height() / qreal(sourceSize.height()));
-        // round up to integer multiples of 1/120 to avoid issues with triggering hidden panels from the edge
-        scale = std::ceil(scale * 120.0) / 120.0;
-        const QPoint relPos((sourceSize.width() / source->scale() - destinationSize.width() / scale) / 2,
-                            (sourceSize.height() / source->scale() - destinationSize.height() / scale) / 2);
+            // In replicating outputs, we don't change the source
+            // Step 1: set the destination mode, if possible
+            const auto bestMode = getBestMode(output.ptr, source);
+            if (!bestMode) {
+                return false;
+            }
+            // Step 2: reposition ans scale destination output to be centered inside the source output
+            auto sourceSize = source->currentMode()->size();
+            auto destinationSize = bestMode->size();
+            if (source->rotation() == KScreen::Output::Left || source->rotation() == KScreen::Output::Right) {
+                sourceSize = sourceSize.transposed();
+            }
+            if (output.ptr->rotation() == KScreen::Output::Left || output.ptr->rotation() == KScreen::Output::Right) {
+                destinationSize = destinationSize.transposed();
+            }
+            qreal scale =
+                source->scale() * std::max(destinationSize.width() / qreal(sourceSize.width()), destinationSize.height() / qreal(sourceSize.height()));
+            // round up to integer multiples of 1/120 to avoid issues with triggering hidden panels from the edge
+            scale = std::ceil(scale * 120.0) / 120.0;
+            const QPoint relPos((sourceSize.width() / source->scale() - destinationSize.width() / scale) / 2,
+                                (sourceSize.height() / source->scale() - destinationSize.height() / scale) / 2);
 
-        output.ptr->setCurrentModeId(bestMode->id());
-        output.ptr->setScale(scale);
-        m_config->setReplicationSource(output.ptr, source);
-        output.posReset = std::optional(output.ptr->pos());
-        output.ptr->setPos(source->pos() + relPos);
+            output.ptr->setCurrentModeId(bestMode->id());
+            output.ptr->setScale(scale);
+            m_config->setReplicationSource(output.ptr, source);
+            output.posReset = std::optional(output.ptr->pos());
+            output.ptr->setPos(source->pos() + relPos);
+        }
     }
 
     reposition();
@@ -948,6 +1006,11 @@ QModelIndex OutputModel::indexForOutputId(int outputId) const
     return QModelIndex();
 }
 
+QModelIndex OutputModel::indexForOutput(const KScreen::OutputPtr &output) const
+{
+    return indexForOutputId(output->id());
+}
+
 bool OutputModel::positionable(const Output &output) const
 {
     return output.ptr->isPositionable();
